@@ -19,12 +19,12 @@
 
 from pydolphinscheduler.core.database import Database
 from pydolphinscheduler.core.engine import Engine
-from pydolphinscheduler.core.process_definition import ProcessDefinition
 from pydolphinscheduler.core.task import Task
+from pydolphinscheduler.core.workflow import Workflow
 
 __all__ = [
     "Database",
     "Engine",
-    "ProcessDefinition",
+    "Workflow",
     "Task",
 ]
