@@ -18,7 +18,7 @@
 # [start workflow_declare]
 """A example workflow for task openmldb."""
 
-from pydolphinscheduler.core.process_definition import ProcessDefinition
+from pydolphinscheduler.core.workflow import Workflow
 from pydolphinscheduler.tasks.openmldb import OpenMLDB
 
 sql = """USE demo_db;
@@ -27,10 +27,10 @@ LOAD DATA INFILE 'file:///tmp/train_sample.csv'
 INTO TABLE talkingdata OPTIONS(mode='overwrite');
 """
 
-with ProcessDefinition(
+with Workflow(
     name="task_openmldb_example",
     tenant="tenant_exists",
-) as pd:
+) as workflow:
     task_openmldb = OpenMLDB(
         name="task_openmldb",
         zookeeper="127.0.0.1:2181",
@@ -39,5 +39,5 @@ with ProcessDefinition(
         sql=sql,
     )
 
-    pd.run()
+    workflow.run()
 # [end workflow_declare]
